// rwt2txt.cpp : convert a starcraft replay with text to a text file
// (the .rep is not modified)

// by superpenguin

// crc codes are not published in this version



#include <windows.h>

#include <stdio.h>
#include <string.h>


// used to convert game ticks to seconds
#define TICK_PER_SEC	23

#define MAGIC_FORMAT_CODE	0x28071982	// common code for all apps
#define MAGIC_APP_CODE		0x52575420	// personnal application code, "RWT " in ascii
#define APP_VERSION			0x00000001	// version, 1 is compatible with the old 0

// describe each DWORD in the header and their order
#define ORDER_SIZE			0
#define ORDER_APP_CODE		1
#define ORDER_APP_VERSION	2
#define ORDER_FORMAT_CODE	3

#define SIZEOF_HEADER		(sizeof(DWORD) * 4)


// check if expected number of bytes have been read
BOOL fread2(char *lpBuffer, DWORD nNumberOfBytesToRead, FILE *file) {
	return (fread(lpBuffer, 1, nNumberOfBytesToRead, file)!=nNumberOfBytesToRead);
}

// get the raw RWT data (header handling)
char *read_repfile(FILE *file, DWORD *version, DWORD *size) {
	char *data=NULL;
	DWORD header[4]={0,0,0,0};

	if (fseek(file, - (long int) SIZEOF_HEADER, SEEK_END)) {
		printf("seek1 failed\n");
		return NULL;
	}
	while (1) {
		if (fread2((char *) header, SIZEOF_HEADER, file)) {
			printf("read1 failed\n");
			return NULL;
		}
		if (header[ORDER_FORMAT_CODE]!=MAGIC_FORMAT_CODE){
			printf("format check failed\n");
			return NULL;
		}
		if (header[ORDER_APP_CODE]!=MAGIC_APP_CODE || HIWORD(header[ORDER_APP_VERSION])>HIWORD(APP_VERSION)) {
			// skip that, rewind to earlier header
			if (fseek(file, -(signed long int)header[ORDER_SIZE]*2-(SIZEOF_HEADER),SEEK_CUR)) {
				printf("seek2 failed\n");
				return NULL;
			}
			continue;
		}
		*version=header[ORDER_APP_VERSION];
		// rewind to this header datas
		if (fseek(file, -(signed long int)header[ORDER_SIZE]-SIZEOF_HEADER, SEEK_CUR)) {
			printf("seek3 failed\n");
			return NULL;
		}
		*size=header[ORDER_SIZE];
		data=(char*)malloc(*size);
		if (fread2(data, *size, file)) {
			printf("read2 failed\n");
			return NULL;
		}
		return data;
	}
	return data;
}


// convert a tick number to a "00:05:10" str for example
// strtime must be an allocated char of at least 9 bytes
void tick2time(char *strtime, DWORD tick) {
	int sec, min, hour;
	tick/=23;
	sec=(int) (tick%60);
	tick/=60;
	min=(int) tick%60;
	tick/=60;
	if (tick>99)
		tick=99; // just in case you like 5 days long games :)
	hour=(int) tick;

	sprintf(strtime, "%02d:%02d:%02d", hour, min, sec);
}

// load the chat in the replay
// writte each line in the new file
void load_chat(char *replaypath, char *txtpath) {
	DWORD version=APP_VERSION;
	char *data;
	DWORD i=0, pos;
	DWORD size;
	FILE *file, *outfile;
	char strtime[9];


	file=fopen(replaypath,"rb");
	if (file==NULL) {
		printf("Cant open replay in read mode: \"%s\"", replaypath);
		return;
	}
	outfile=fopen(txtpath, "w");
	if (file==NULL) {
		printf("Cant open outputfile: \"%s\"", txtpath);
		return;
	}

	if ((data=read_repfile(file,&version, &size))==NULL) {
		printf("Cant read or detect chatlog in this replay: \"%s\"", replaypath);
		fclose(file);
		return;
	}

	if (data[size-1]!=0) {
		printf("Invalid end of textdata in replay: \"%s\"", replaypath);
	} else {
		while (i+sizeof(DWORD)+1<size &&  *((DWORD *) (data+i))!=0xFFFFFFFF) {
			pos=i;
			tick2time(strtime, *((DWORD *) (data+pos)));
			fprintf(outfile, "%s %s\n",strtime,(char *) (data+pos+sizeof(DWORD)));
			i+=sizeof(DWORD);
			for (; i<size && data[i]!=0; i++);
			i++;
		}
	}
	fclose(file);
	fclose(outfile);
	printf("text file generated\n");
}

int main(int argc, char* argv[])
{
	char txtpath[MAX_PATH];
	if (argc!=2) {
		printf("you must use 1 argument : the replay name");
		return 1;
	}
	if (strlen(argv[1]) + 4 >= MAX_PATH) {
		printf("path too long");
		return 1;
	}

	strncpy(txtpath, argv[1], MAX_PATH);
	strcat(txtpath, ".txt");

	load_chat(argv[1], txtpath);

	return 0;
}

